# -*- coding: utf-8 -*-

from collective.compoundcriterion.interfaces import ICompoundCriterionFilter

from plone import api

from imio.schedule.content.schedule_config import IScheduleConfig
from imio.schedule.content.task_config import ITaskConfig
from imio.schedule.interfaces import IToTaskConfig
from imio.schedule.interfaces import TaskConfigNotFound
from imio.schedule.utils import interface_to_tuple
from imio.schedule.utils import tuple_to_interface

from zope.component import getGlobalSiteManager
from zope.interface import Interface
from zope.interface import implements


import logging

logger = logging.getLogger('schedule')


def subscribe_task_configs_for_content_type(task_config, event):
    """
    Register adapter returning 'task_config' to the interface
    of the content_type selected in the field 'task_container'.
    """

    gsm = getGlobalSiteManager()
    task_config_UID = task_config.UID()

    class TaskConfigSubscriber(object):
        """ """
        implements(IToTaskConfig)

        def __init__(self, context):
            """ """
            self.context = context
            self.task_config_UID = task_config_UID

        @property
        def task_config(self):
            catalog = api.portal.get_tool('portal_catalog')
            brains = catalog.unrestrictedSearchResults(UID=self.task_config_UID)
            if brains:
                task_config = brains[0].getObject()
                return task_config
            else:
                raise TaskConfigNotFound(
                    'UID {}'.format(self.task_config_UID)
                )

    registration_interfaces = task_config.get_scheduled_interfaces()

    for registration_interface in registration_interfaces:
        gsm.registerAdapter(
            factory=TaskConfigSubscriber,
            required=(registration_interface,),
            provided=IToTaskConfig,
            name=task_config.UID()
        )
        msg = "Registered IToTaskConfig adapter '{}' for {}".format(
            task_config.Title(),
            registration_interface
        )
        logger.info(msg)


def unsubscribe_task_configs_for_content_type(task_config, event):
    """
    Unregister adapter returning 'task_config' to the interface
    of the content_type selected in the field 'task_container'.
    """

    gsm = getGlobalSiteManager()
    schedule_config = task_config.get_schedule_config()

    previous_interfaces = getattr(schedule_config, '_scheduled_interface_', None)
    if previous_interfaces and type(previous_interfaces[0]) not in [list, tuple]:
        previous_interfaces = (previous_interfaces,)
    previous_interfaces = [tuple_to_interface(i) for i in previous_interfaces]

    for previous_interface in previous_interfaces:
        removed = gsm.unregisterAdapter(
            required=(previous_interface,),
            provided=IToTaskConfig,
            name=task_config.UID()
        )
        if removed:
            msg = "Unregistered IToTaskConfig adapter '{}' for {}".format(
                task_config.Title(),
                previous_interface
            )
            logger.info(msg)


def update_task_configs_subscriptions(schedule_config, event):
    """
    When the scheduled_contenttype value of a ScheduleConfig is changed,
    we have to unregister all the adapters providing IToTaskConfig
    and register them again for the new selected content type.
    """

    previous_interfaces = getattr(schedule_config, '_scheduled_interface_', None)
    new_interfaces = schedule_config.get_scheduled_interfaces()
    new_interfaces = tuple([interface_to_tuple(i) for i in new_interfaces])

    # if there were no previous values, just save it and return
    if not previous_interfaces:
        setattr(schedule_config, '_scheduled_interface_', new_interfaces)
        return

    # if the value did not change, do nothing
    if previous_interfaces == new_interfaces:
        return

    for task_config in schedule_config.get_all_task_configs():
        # unregister the IToTaskConfig adapter for the previous interface
        unsubscribe_task_configs_for_content_type(task_config, event)
        # register the new IToTaskConfig adapter for the new interface
        subscribe_task_configs_for_content_type(task_config, event)

    # replace the _schedule_interface_ attribute with the new value
    setattr(schedule_config, '_scheduled_interface_', new_interfaces)


def register_schedule_collection_criterion(schedule_config, event):
    """
    Register adapter turning a schedule config into a collection
    criterion filtering all tasks from this schedule config.
    """

    gsm = getGlobalSiteManager()
    schedule_config_UID = schedule_config.UID()

    class FilterTasksCriterion(object):

        def __init__(self, context):
            self.context = context

        @property
        def query(self):
            return {'schedule_config_UID': {'query': schedule_config_UID}}

    gsm.registerAdapter(
        factory=FilterTasksCriterion,
        required=(Interface,),
        provided=ICompoundCriterionFilter,
        name=schedule_config.UID()
    )
    msg = "Registered schedule CollectionCriterion adapter '{}'".format(
        schedule_config.Title()
    )
    logger.info(msg)


def unregister_schedule_collection_criterion(schedule_config, event):
    """
    Unregister adapter turning a schedule config into a collection
    criterion.
    """

    gsm = getGlobalSiteManager()

    removed = gsm.unregisterAdapter(
        required=(Interface,),
        provided=ICompoundCriterionFilter,
        name=schedule_config.UID()
    )
    if removed:
        msg = "Unregistered schedule CollectionCriterion adapter '{}'".format(
            schedule_config.Title()
        )
        logger.info(msg)


def register_task_collection_criterion(task_config, event):
    """
    Register adapter turning a task config into a collection
    criterion filtering all tasks from this task config.
    """

    gsm = getGlobalSiteManager()
    task_config_UID = task_config.UID()

    class FilterTasksCriterion(object):

        def __init__(self, context):
            self.context = context

        @property
        def query(self):
            return {'task_config_UID': {'query': task_config_UID}}

    gsm.registerAdapter(
        factory=FilterTasksCriterion,
        required=(Interface,),
        provided=ICompoundCriterionFilter,
        name=task_config.UID()
    )
    msg = "Registered task CollectionCriterion adapter '{}'".format(
        task_config.Title()
    )
    logger.info(msg)


def unregister_task_collection_criterion(task_config, event):
    """
    Unregister adapter turning a task config into a collection
    criterion.
    """

    gsm = getGlobalSiteManager()

    removed = gsm.unregisterAdapter(
        required=(Interface,),
        provided=ICompoundCriterionFilter,
        name=task_config.UID()
    )
    if removed:
        msg = "Unregistered task CollectionCriterion adapter '{}'".format(
            task_config.Title()
        )
        logger.info(msg)


_registered_sites = set()


def register_at_instance_startup(site, event):
    """
    Re-register:
        - all the TaskConfig adapters
        - collections criterions
        - tasks vocabulary of each ScheduleConfig
    when zope instance is started.
    """
    if site.id not in _registered_sites:

        # register task configs subscribers and task configs criterion
        catalog = api.portal.get_tool('portal_catalog')
        task_brains = catalog.unrestrictedSearchResults(object_provides=ITaskConfig.__identifier__)
        all_task_configs = [site.unrestrictedTraverse(brain.getPath()) for brain in task_brains]

        for task_config in all_task_configs:
            subscribe_task_configs_for_content_type(task_config, event)
            register_task_collection_criterion(task_config, event)

        # register schedule configs criterion and tasks vocabulary
        schedule_brains = catalog.unrestrictedSearchResults(object_provides=IScheduleConfig.__identifier__)
        all_schedule_configs = [site.unrestrictedTraverse(brain.getPath()) for brain in schedule_brains]

        for schedule_config in all_schedule_configs:
            register_schedule_collection_criterion(schedule_config, event)

        _registered_sites.add(site.id)



# -*- coding: utf-8 -*-

from plone import api
from plone.registry import Record
from plone.registry import field
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.interface import alsoProvides


def upgrade_2_fix_schedule_collection(context):
    from collective.eeafaceted.collectionwidget.interfaces import IDashboardCollection
    from imio.schedule.interfaces import IScheduleCollection

    brains = api.content.find(
        object_provides=IDashboardCollection.__identifier__,
        id="dashboard_collection",
    )
    for brain in brains:
        if "schedule" not in brain.getPath():
            continue
        collection = brain.getObject()
        alsoProvides(collection, IScheduleCollection)


def upgrade_3_set_showNumberOfItems(context):
    from collective.eeafaceted.collectionwidget.interfaces import IDashboardCollection
    brains = api.content.find(
        object_provides=IDashboardCollection.__identifier__,
        id="dashboard_collection",
    )
    for brain in brains:
        if "schedule" not in brain.getPath():
            continue
        collection = brain.getObject()
        collection.showNumberOfItems = True


def upgrade_4_add_due_date_reminders(context):
    from imio.schedule.interfaces import IDueDateSettings

    setup_tool = api.portal.get_tool("portal_setup")
    registry = getUtility(IRegistry)

    default_values = {
        'color_orange_x_days_before_due_date': 10,
        'color_red_x_days_before_due_date': 5,
    }

    base = 'imio.schedule.interfaces.IDueDateSettings'
    for key, default_value in default_values.items():
        full_key = "{0}.{1}".format(base, key)
        if full_key not in registry.records:
            registry_field = field.Int(title=IDueDateSettings[key].title, required=False, min=0)
            registry_record = Record(registry_field)
            registry_record.value = default_value
            registry.records[full_key] = registry_record



# -*- coding: utf-8 -*-
"""
imio.schedule
-------------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from imio.schedule import utils
from imio.schedule import _


class WorkingDaysVocabularyFactory(object):

    def __call__(self, context):
        return utils.dict_list_2_vocabulary([
            {'monday': _(u'Monday')},
            {'tuesday': _(u'Tuesday')},
            {'wednesday': _(u'Wednesday')},
            {'thursday': _(u'Thursday')},
            {'friday': _(u'Friday')},
            {'saturday': _(u'Saturday')},
            {'sunday': _(u'Sunday')},
        ])


class AdditionalDelayTypeVocabularyFactory(object):

    def __call__(self, context):
        return utils.dict_list_2_vocabulary([
            {'absolute': _(u'Absolute')},
            {'working_days': _(u'Working days')},
        ])



# -*- coding: utf-8 -*-

from Acquisition import aq_base

from imio.schedule.interfaces import TaskAlreadyExists
from imio.schedule.utils import get_task_configs

from plone import api


def create_new_tasks(task_container, event):
    """
    For each task config associated to this task container content type
    check the task config creations conditions and create the task if
    we can.
    """

    # This handler can be triggered for archetypes containers by the
    # workflow modification event but we want to create tasks only if
    # the container really exists (more than just created in portal_factory...)
    if hasattr(aq_base(task_container), 'checkCreationFlag'):
        if task_container.checkCreationFlag():
            return

    # descending=True <= it's important to create to macro tasks first
    task_configs = get_task_configs(task_container, descending=True)

    if not task_configs:
        return

    with api.env.adopt_roles(['Manager']):
        for config in task_configs:
            if config.is_main_taskconfig():
                if config.should_create_task(task_container):
                    try:
                        config.create_task(task_container)
                    except TaskAlreadyExists:
                        continue
            # case of a sub-task creation, the parent should have been created first
            else:
                macro_config = config.getParentNode()
                parent_task = macro_config.get_open_task(task_container)
                if parent_task and config.should_create_task(task_container, parent_container=parent_task):
                    try:
                        config.create_task(task_container, creation_place=parent_task)
                    except TaskAlreadyExists:
                        continue



# -*- coding: utf-8 -*-

from Acquisition import aq_base

from imio.schedule.testing import ExampleScheduleIntegrationTestCase
from imio.schedule.testing import TEST_INSTALL_INTEGRATION

from plone import api

import unittest


class TestScheduleConfig(unittest.TestCase):
    """
    Test ScheduleConfig content type.
    """

    layer = TEST_INSTALL_INTEGRATION

    def test_ScheduleConfig_portal_type_is_registered(self):
        portal_types = api.portal.get_tool('portal_types')
        registered_types = portal_types.listContentTypes()
        self.assertTrue('ScheduleConfig' in registered_types)


class TestScheduleConfigFields(ExampleScheduleIntegrationTestCase):
    """
    Test schema fields declaration.
    """

    def test_class_registration(self):
        """
        Check if the class of the content type ScheduleConfig is the
        correct one.
        """
        from imio.schedule.content.schedule_config import ScheduleConfig
        self.assertTrue(self.schedule_config.__class__ == ScheduleConfig)

    def test_schema_registration(self):
        """
        Check if the schema Interface of the content type ScheduleConfig is the
        correct one.
        """
        portal_types = api.portal.get_tool('portal_types')
        scheduleconfig_type = portal_types.get(self.schedule_config.portal_type)
        self.assertTrue('IScheduleConfig' in scheduleconfig_type.schema)

    def test_enabled_attribute(self):
        task_config = aq_base(self.task_config)
        self.assertTrue(hasattr(task_config, 'enabled'))

    def test_enabled_field_display(self):
        self.browser.open(self.task_config.absolute_url())
        contents = self.browser.contents
        msg = "field 'enabled' is not displayed"
        self.assertTrue('id="form-widgets-enabled"' in contents, msg)
        msg = "field 'enabled' is not translated"
        self.assertTrue('Activé' in contents, msg)

    def test_enabled_field_edit(self):
        self.browser.open(self.task_config.absolute_url() + '/edit')
        contents = self.browser.contents
        msg = "field 'enabled' is not editable"
        self.assertTrue('Activé' in contents, msg)

    def test_scheduled_contenttype_attribute(self):
        schedule_config = aq_base(self.schedule_config)
        self.assertTrue(hasattr(schedule_config, 'scheduled_contenttype'))

    def test_scheduled_contenttype_field_display(self):
        self.browser.open(self.schedule_config.absolute_url())
        contents = self.browser.contents
        msg = "field 'scheduled_contenttype' is not displayed"
        self.assertTrue('id="form-widgets-scheduled_contenttype"' in contents, msg)
        msg = "field 'scheduled_contenttype' is not translated"
        self.assertTrue('Type de contenu associé' in contents, msg)

    def test_scheduled_contenttype_field_edit(self):
        self.browser.open(self.schedule_config.absolute_url() + '/edit')
        contents = self.browser.contents
        msg = "field 'scheduled_contenttype' is not editable"
        self.assertTrue('Type de contenu associé' in contents, msg)


class TestScheduleConfigIntegration(ExampleScheduleIntegrationTestCase):
    """
    Test ScheduleConfig methods.
    """

    def test_get_all_task_configs(self):
        """
        Should return all TaskConfig contained in the ScheduleConfig.
        """
        expected_taskconfigs = [self.task_config]
        task_configs = self.schedule_config.get_all_task_configs()
        msg = 'expected {} but got {}'.format(expected_taskconfigs, task_configs)
        self.assertTrue(task_configs == expected_taskconfigs, msg)

    def test_get_scheduled_portal_type(self):
        """
        Should return the portal_type of the content type selected on the field
        'scheduled_contenttype'.
        """
        portal_type = self.schedule_config.get_scheduled_portal_type()
        expected_type = 'Folder'
        msg = 'expected {} but got {}'.format(expected_type, portal_type)
        self.assertTrue(portal_type == expected_type, msg)

    def test_get_scheduled_interfaces(self):
        """
        Should return the Interface (or a class) of the content type selected
        on the field 'scheduled_contenttype'.
        """
        from Products.ATContentTypes.interfaces import IATFolder

        type_interface = self.schedule_config.get_scheduled_interfaces()
        expected_interface = (IATFolder,)
        msg = 'expected {} but got {}'.format(expected_interface, type_interface)
        self.assertTrue(type_interface == expected_interface, msg)

    def test_is_empty(self):
        """
        Should return True if the schedule_config has no TaskConfig.
        """
        self.assertTrue(not self.schedule_config.is_empty())
        self.assertTrue(self.empty_schedule_config.is_empty())



# -*- coding: utf-8 -*-

from collective.eeafaceted.collectionwidget.vocabulary import CollectionVocabulary

from plone import api

from zope.annotation import IAnnotations


class ScheduleCollectionVocabulary(CollectionVocabulary):
    """
    Return vocabulary of base searchs for schedule faceted view.
    """

    def _brains(self, context):
        """
        Return all the DashboardCollections in the 'schedule' folder.
        """
        configs_UID = IAnnotations(context).get('imio.schedule.schedule_configs', [])
        catalog = api.portal.get_tool('portal_catalog')
        config_brains = catalog(UID=configs_UID)
        collections_brains = []
        for brain in config_brains:
            config = brain.getObject()
            brains = catalog(
                path={
                    'query': '/'.join(config.getPhysicalPath()),
                },
                object_provides='plone.app.contenttypes.interfaces.ICollection',
                sort_on='getObjPositionInParent'
            )
            collections_brains.extend(brains)
        collections_brains = [b for b in collections_brains if b.getObject().aq_parent.enabled]
        return collections_brains

ScheduleCollectionVocabularyFactory = ScheduleCollectionVocabulary()



# -*- coding: utf-8 -*-

from imio.schedule.testing import NAKED_PLONE_INTEGRATION
from imio.schedule.testing import TEST_INSTALL_INTEGRATION  # noqa

from plone import api
from plone.app.testing import applyProfile

import unittest


class TestInstallDependencies(unittest.TestCase):

    layer = NAKED_PLONE_INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_dexterity_is_dependency_of_imio_schedule(self):
        """
        dexterity should be installed when we install imio.schedule
        """
        self.assertTrue(not self.installer.isProductInstalled('plone.app.dexterity'))
        applyProfile(self.portal, 'imio.schedule:default')
        self.assertTrue(self.installer.isProductInstalled('plone.app.dexterity'))

    def test_z3cformdatagridfield_is_dependency_of_imio_schedule(self):
        """
        z3cform.datagridfield should be installed when we install imio.schedule
        """
        self.assertTrue(not self.installer.isProductInstalled('collective.z3cform.datagridfield'))
        applyProfile(self.portal, 'imio.schedule:default')
        self.assertTrue(self.installer.isProductInstalled('collective.z3cform.datagridfield'))

    def test_collectivetask_is_dependency_of_imio_schedule(self):
        """
        collective.task should be installed when we install imio.schedule
        """
        self.assertTrue(not self.installer.isProductInstalled('collective.task'))
        applyProfile(self.portal, 'imio.schedule:default')
        self.assertTrue(self.installer.isProductInstalled('collective.task'))


class TestSetup(unittest.TestCase):
    """
    Test that imio.schedule is properly installed.
    """

    layer = TEST_INSTALL_INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """
        Test if imio.schedule is installed.
        """
        self.assertTrue(self.installer.isProductInstalled('imio.schedule'))

    def test_browserlayer(self):
        """
        Test that IImioScheduleLayer is registered.
        """
        from imio.schedule.interfaces import IImioScheduleLayer
        from plone.browserlayer import utils
        self.assertIn(IImioScheduleLayer, utils.registered_layers())

    def test_testing_profile_is_registered(self):
        """
        Test testing profile is registered.
        """
        portal_setup = api.portal.get_tool(name='portal_setup')
        demo_profile_name = u'imio.schedule:testing'
        profile_ids = [info['id'] for info in portal_setup.listProfileInfo()]
        msg = 'testing profile is not registered'
        self.assertTrue(demo_profile_name in profile_ids, msg)


class TestUninstall(unittest.TestCase):

    layer = TEST_INSTALL_INTEGRATION

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['imio.schedule'])

    def test_product_uninstalled(self):
        """Test if imio.schedule is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled(
            'imio.schedule'))



# -*- coding: utf-8 -*-

from datetime import datetime
from dateutil.relativedelta import relativedelta

from imio.schedule.config import CREATION
from imio.schedule.interfaces import ScheduleConfigNotFound
from imio.schedule.interfaces import TaskConfigNotFound
from imio.schedule.testing import ExampleScheduleFunctionalTestCase
from imio.schedule.testing import ExampleScheduleIntegrationTestCase
from imio.schedule.testing import MacroTaskScheduleIntegrationTestCase
from imio.schedule.testing import TEST_INSTALL_INTEGRATION

from plone import api

import unittest


class TestAutomatedTask(unittest.TestCase):
    """
    Test AutomatedTask content type.
    """

    layer = TEST_INSTALL_INTEGRATION

    def test_AutomatedTask_portal_type_is_registered(self):
        portal_types = api.portal.get_tool('portal_types')
        registered_types = portal_types.listContentTypes()
        self.assertTrue('AutomatedTask' in registered_types)


class TestAutomatedTaskFields(ExampleScheduleIntegrationTestCase):
    """
    Test schema fields declaration.
    """

    def test_class_registration(self):
        """
        Check if the class of the content type AutomatedTask is the
        correct one.
        """
        from imio.schedule.content.task import AutomatedTask
        self.assertTrue(self.task.__class__ == AutomatedTask)

    def test_schema_registration(self):
        """
        Check if the schema Interface of the content type AutomatedTask is the
        correct one.
        """
        portal_types = api.portal.get_tool('portal_types')
        taskconfig_type = portal_types.get(self.task.portal_type)
        self.assertTrue('IAutomatedTask' in taskconfig_type.schema)


class TestAutomatedTaskIntegration(ExampleScheduleFunctionalTestCase):
    """
    Test AutomatedTask methods.
    """

    def test_get_container(self):
        """
        Should return the task container of the task.
        """
        task_container = self.task.get_container()
        expected_container = self.task_container
        self.assertEquals(task_container, expected_container)

    def test_get_schedule_config(self):
        """
        Should return the schedule config of the task.
        """
        task = self.task

        schedule_config = task.get_schedule_config()
        expected_config = self.schedule_config
        self.assertEquals(schedule_config, expected_config)

        api.content.delete(self.schedule_config)
        with self.assertRaises(ScheduleConfigNotFound):
            task.get_schedule_config()

    def test_get_task_config(self):
        """
        Should return the associated TaskConfig.
        """
        task = self.task

        config = task.get_task_config()
        expected_config = self.task_config
        self.assertEquals(config, expected_config)

        api.content.delete(self.task_config)
        with self.assertRaises(TaskConfigNotFound):
            task.get_task_config()

    def test_get_status(self):
        """
        Should return CREATION, STARTED or DONE depending
        on the task workflow state.
        """
        task = self.task
        expected_status = CREATION
        self.assertEquals(task.get_status(), expected_status)

    def test_start_conditions_status(self):
        """
        Check that the implementation is delegated to the
        'start_conditions_status' method of the task config.
        """
        task_config = self.task_config
        task_container = self.task_container
        task = self.task

        expected_status = task_config.start_conditions_status(task_container, task)
        status = task.start_conditions_status()

        msg = "implementation should just call the method 'start_conditions_status' from the task config"
        self.assertTrue(expected_status == status, msg)

    def test_starting_states_status(self):
        """
        Should return the following tuple:
        (task_container state, [starting states of the task])
        """
        task_config = self.task_config
        task_container = self.task_container
        task = self.task

        task_container_state, starting_states = task.starting_states_status()
        self.assertEquals(task_container_state, api.content.get_state(task_container))
        self.assertEquals(starting_states, task_config.starting_states)

    def test_end_conditions_status(self):
        """
        Check that the implementation is delegated to the
        'end_conditions_status' method of the task config.
        """
        task_config = self.task_config
        task_container = self.task_container
        task = self.task

        expected_status = task_config.end_conditions_status(task_container, task)
        status = task.end_conditions_status()

        msg = "implementation should just call the method 'end_conditions_status' from the task config"
        self.assertTrue(expected_status == status, msg)

    def test_ending_states_status(self):
        """
        Should return the following tuple:
        (task_container state, [ending states of the task])
        """
        task_config = self.task_config
        task_container = self.task_container
        task = self.task

        task_container_state, ending_states = task.ending_states_status()
        self.assertEquals(task_container_state, api.content.get_state(task_container))
        self.assertEquals(ending_states, task_config.ending_states)

    def test_get_state(self):
        """
        Should return the current workflow state of the task.
        """
        task = self.task
        self.assertEquals(task.get_state(), api.content.get_state(task))

    def test_get_subtasks(self):
        """
        Should return an empty list since a basic task has no subtasks.
        """
        task = self.task
        self.assertEquals(task.get_subtasks(), [])

    def test_end_date(self):
        """
        Should return the date of the ending of the task.
        """
        task = self.task
        task_config = self.task_config

        self.assertEquals(task.end_date, None)

        task_config.start_task(task)
        self.assertEquals(task.end_date, None)

        task_config.end_task(task)
        end_date = task.end_date
        expected_date = datetime.now(end_date.tzinfo)
        self.assertTrue(relativedelta(expected_date, end_date).seconds < 1)

    def test_start(self):
        """
        Should start a task.
        """
        task = self.task
        msg = "task should not be started yet (for the sake of the test..)"
        self.assertEquals(api.content.get_state(task), 'created', msg)
        task._start()
        msg = "task should have been started"
        self.assertEquals(api.content.get_state(task), 'to_do', msg)

    def test_end(self):
        """
        Should end a task.
        """
        task = self.task
        msg = "task should not be ended yet (for the sake of the test..)"
        self.assertEquals(api.content.get_state(task), 'created', msg)
        task._end()
        msg = "task should have been ended"
        self.assertEquals(api.content.get_state(task), 'closed', msg)

    def test_freeze(self):
        """
        Should freeze a task.
        """
        task = self.task
        msg = "task should not be frozen yet (for the sake of the test..)"
        self.assertEquals(api.content.get_state(task), 'created', msg)
        task._freeze()
        msg = "task should have been frozen"
        self.assertEquals(api.content.get_state(task), 'frozen', msg)

    def test_thaw(self):
        """
        Should thaw a task.
        """
        task = self.task
        msg = "task should be in created state"
        self.assertEquals(api.content.get_state(task), 'created', msg)
        # freeze task befor thaw tests
        task._freeze()
        msg = "task should not be thawed yet (for the sake of the test..)"
        self.assertEquals(api.content.get_state(task), 'frozen', msg)

        task._thaw()

        msg = "task should have been thawed and put back in its original state"
        self.assertEquals(api.content.get_state(task), 'created', msg)


class TestAutomatedMacroTask(unittest.TestCase):
    """
    Test AutomatedMacroTask content type.
    """

    layer = TEST_INSTALL_INTEGRATION

    def test_AutomatedMacroTask_portal_type_is_registered(self):
        portal_types = api.portal.get_tool('portal_types')
        registered_types = portal_types.listContentTypes()
        self.assertTrue('AutomatedMacroTask' in registered_types)


class TestAutomatedMacroTaskFields(MacroTaskScheduleIntegrationTestCase):
    """
    Test schema fields declaration.
    """

    def test_class_registration(self):
        """
        Check if the class of the content type AutomatedMacroTask is the
        correct one.
        """
        from imio.schedule.content.task import AutomatedMacroTask
        self.assertTrue(self.macro_task.__class__ == AutomatedMacroTask)

    def test_schema_registration(self):
        """
        Check if the schema Interface of the content type AutomatedMacroTask is the
        correct one.
        """
        portal_types = api.portal.get_tool('portal_types')
        taskconfig_type = portal_types.get(self.macro_task.portal_type)
        self.assertTrue('IAutomatedTask' in taskconfig_type.schema)


class TestAutomatedMacroTaskIntegration(MacroTaskScheduleIntegrationTestCase):
    """
    Test AutomatedMacroTask methods.
    """

    def test_get_container(self):
        """
        Should return the task container of the task.
        """
        task_container = self.sub_task.get_container()
        expected_container = self.task_container
        self.assertEquals(task_container, expected_container)

    def test_get_task_config(self):
        """
        Should return the associated MacroTaskConfig.
        """
        config = self.macro_task.get_task_config()
        expected_config = self.macrotask_config
        self.assertEquals(config, expected_config)

    def test_get_subtasks(self):
        """
        Should return the subtasks of this macro task.
        """
        subtasks = self.macro_task.get_subtasks()
        self.assertEquals(len(subtasks), 1)
        self.assertEquals(subtasks[0], self.sub_task)

    def test_get_last_subtasks(self):
        """
        Should return the unique subtasks of this macro task.
        """
        subtasks = self.macro_task.get_subtasks()
        self.assertEquals(len(subtasks), 1)
        self.assertEquals(subtasks[0], self.sub_task)



# -*- coding: utf-8 -*-

from collective.compoundcriterion.interfaces import ICompoundCriterionFilter

from plone import api

from Products.ATContentTypes.interfaces import IATDocument

from imio.schedule.interfaces import IToTaskConfig
from imio.schedule.testing import ExampleScheduleFunctionalTestCase
from imio.schedule.utils import interface_to_tuple

from zope.component import getAdapter
from zope.component import queryAdapter
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent


class TestIToTaskConfigAdaptersRegistration(ExampleScheduleFunctionalTestCase):
    """
    Test the automated IToTaskConfig adapters registration.
    """

    def test_registration_at_instance_start(self):
        """
        Sice we can not simulate a instance restart in the tests
        Just check that we when through the event handler by checking
        the var  '_registered_sites'.
        """
        from imio.schedule.events import zope_registration

        self.assertTrue(len(zope_registration._registered_sites) > 0)

    def test_registration_on_TaskConfig_creation(self):
        """
        When creating a new TaskConfig,
        a new adapter should be registered with:
            for: ATFolder (type selected on the ScheduleConfig of this TaskConfig)
            provides: IToTaskConfig
            name: this TaskConfig UID
        """

        task_config = api.content.create(
            type='TaskConfig',
            id='task_config_2',
            container=self.schedule_config
        )

        folder = self.portal.config
        adapter = queryAdapter(folder, IToTaskConfig, task_config.UID())
        msg = "an adapter should have been registered when creating a new TaskConfig"
        self.assertTrue(adapter is not None, msg)

        # delete the task config and unregister its adapter manually since tests layer
        # tearDown cannot take care of this
        api.content.delete(task_config)

    def test_unregistration_on_TaskConfig_deletion(self):
        """
        When deleting a TaskConfig, its IToTaskConfig adapter should be unregistered
        """

        task_config = api.content.create(
            type='TaskConfig',
            id='task_config_2',
            container=self.schedule_config
        )

        # to test unregistration, we have to be sure something was registered
        folder = self.portal.config
        adapter = getAdapter(folder, IToTaskConfig, task_config.UID())
        msg = "an adapter providing IToTaskConfig should have been registered for IATFolder"
        self.assertTrue(adapter is not None, msg)

        api.content.delete(task_config)
        adapter = queryAdapter(folder, IToTaskConfig, task_config.UID())
        msg = "the adapter should have been unregistered when deleting the TaskConfig"
        self.assertTrue(adapter is None, msg)

    def test_adapters_update_on_LicenceSchedule_modification(self):
        """
        When an other content type is selected on the field 'scheduled_contenttype'
        of a ScheduleConfig, we have to unregister all the IToTaskConfig adapter of
        each TaskConfig of this ScheduleConfig and register them back for the new
        portal_type.
        """
        schedule_config = self.schedule_config
        task_config = self.task_config
        folder = self.portal.config
        document = api.content.create(type='Document', id='doc', container=self.portal)

        # the adapter should be registered for IATFolder
        adapter = getAdapter(folder, IToTaskConfig, task_config.UID())
        msg = "an adapter providing IToTaskConfig should have been registered for IATFolder"
        self.assertTrue(adapter is not None, msg)

        #... but not for IATDocument
        adapter = queryAdapter(document, IToTaskConfig, task_config.UID())
        msg = "not adapter should have been registered for IATDocument yet..."
        self.assertTrue(adapter is None, msg)

        # modify 'scheduled_contenttype' then manually trigger the modification event
        schedule_config.scheduled_contenttype = ('Document', interface_to_tuple(IATDocument))
        notify(ObjectModifiedEvent(schedule_config))

        # old IToTaskConfig adapter should be unregistered for IATFolder
        adapter = queryAdapter(folder, IToTaskConfig, task_config.UID())
        msg = "the adapter should have been unregistered when modifying the ScheduleConfig"
        self.assertTrue(adapter is None, msg)

        # new IToTaskConfig adapter should be registered for IATDocument
        adapter = queryAdapter(document, IToTaskConfig, task_config.UID())
        msg = "an adapter providing IToTaskConfig should have been registered for IATDocument"
        self.assertTrue(adapter is not None, msg)


class TestDashboardCriterionRegistration(ExampleScheduleFunctionalTestCase):
    """
    Test the automated registration of task filter criterion and
    its vocabulary.
    """

    def test_criterion_registration_on_ScheduleConfig_creation(self):
        """
        When creating a new ScheduleConfig,
        A vocabulary, named with the ScheduleConfig title, listing all its subtasks
        should be registered.
        """
        adapter = queryAdapter(
            self.portal,
            ICompoundCriterionFilter,
            self.schedule_config.UID()
        )
        msg = "an criterion adapter should have been registered when creating a new ScheduleConfig"
        self.assertTrue(adapter is not None, msg)

    def test_criterion_unregistration_on_ScheduleConfig_deletion(self):
        """
        When deleting a ScheduleConfig, its IVocabularyFactory should be
        unregistered.
        """
        schedule_config = api.content.create(
            type='ScheduleConfig',
            id='schedule_config_2',
            container=self.portal.config,
            scheduled_contenttype=self.schedule_config.scheduled_contenttype,
        )

        # to test unregistration, we have to be sure something was registered
        adapter = queryAdapter(self.portal, ICompoundCriterionFilter, schedule_config.UID())
        msg = "an adapter providing ICompoundCriterionFilter should have been registered"
        self.assertTrue(adapter is not None, msg)

        schedule_config_UID = schedule_config.UID()
        api.content.delete(schedule_config)
        adapter = queryAdapter(self.portal, ICompoundCriterionFilter, schedule_config_UID)
        msg = "the ICompoundCriterionFilter adapter should have been unregistered when deleting the ScheduleConfig"
        self.assertTrue(adapter is None, msg)



# -*- coding: utf-8 -*-

from imio.schedule.content.logic import AssignTaskGroup

from plone import api


class AssignAuthenticatedUsersGroup(AssignTaskGroup):
    """
    Return the AuthenticatedUsers group as the default group
    to assign on a new AutomatedTask.
    """

    def group_id(self):
        """
        Return the id of AuthenticatedUsers group.
        """
        return api.group.get('AuthenticatedUsers')
